#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"

#define SCREEN_WIDTH    640
#define SCREEN_HEIGHT   480
#define MAX_ITERATIONS  256

void drawMandelbrot(int offsetx, int offsety)
{
    for (int y = 0; y < SCREEN_HEIGHT; y += 8)
    {
        for (int x = 0; x < SCREEN_WIDTH; x += 8)
        {
            // convert the pixel pos to a complex number
            double c_re = (x + offsetx - SCREEN_WIDTH  / 2.0) * 4.0 / SCREEN_WIDTH - 0.7;
            double c_im = (y + offsety - SCREEN_HEIGHT / 2.0) * 4.0 / SCREEN_WIDTH;

            double z_re = 0;
            double z_im = 0;
            int iteration = 0;

            while (z_re * z_re + z_im * z_im <= 16 &&
                   iteration < MAX_ITERATIONS)
            {
                // compute the new value
                double zn_re = z_re * z_re - z_im * z_im + c_re;
                double zn_im = 2 * z_re * z_im + c_im;

                // copy back the result to z
                z_re = zn_re;
                z_im = zn_im;

                iteration++;
            }

            // draw the pixel
            Color col;

            if (iteration == MAX_ITERATIONS)
                col = Color(0, 0, 0);
            else
                col = Color(iteration, iteration, iteration/2 + 128);

            gfx.setPixel(x + offsetx, y + offsety, col);
        }
    }
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Mandelbrot 2", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    static Uint8 pattern[64] =
    {
         0, 32,  8, 40,  2, 34, 10, 42,
        48, 16, 56, 24, 50, 18, 58, 26,
        12, 44,  4, 36, 14, 46,  6, 38,
        60, 28, 52, 20, 62, 30, 54, 22,
         3, 35, 11, 43,  1, 33,  9, 41,
        51, 19, 59, 27, 49, 17, 57, 25,
        15, 47,  7, 39, 13, 45,  5, 37,
        63, 31, 55, 23, 61, 29, 53, 21
    };

    for (int i = 0; i < 64; i++)
    {
        int j = 0;
        while (pattern[j] != i)
            j++;

        drawMandelbrot(j % 8, j / 8);

        // display and process the events
        gfx.render();
        sys.wait(50);
        sys.processEvents();
        if (sys.isQuitRequested() == true)
            exit(EXIT_SUCCESS);
    }

    // wait until we quit
    while (sys.isQuitRequested() == false)
    {
        sys.processEvents();
        sys.wait(50);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
